package com.tzld.piaoquan.recommend.server.config;

import org.apache.commons.pool2.impl.GenericObjectPoolConfig;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Primary;
import org.springframework.data.redis.connection.RedisConnectionFactory;
import org.springframework.data.redis.connection.RedisStandaloneConfiguration;
import org.springframework.data.redis.connection.lettuce.LettuceClientConfiguration;
import org.springframework.data.redis.connection.lettuce.LettuceConnectionFactory;
import org.springframework.data.redis.connection.lettuce.LettucePoolingClientConfiguration;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.serializer.StringRedisSerializer;

@Configuration
public class VideoRedisTemplateConfig {

    @Bean("videoRedisPool")
    @ConfigurationProperties(prefix = "spring.video-redis.lettuce.pool")
    public GenericObjectPoolConfig<LettucePoolingClientConfiguration> videoRedisPool() {
        return new GenericObjectPoolConfig<>();
    }

    @Bean("videoRedisConfig")
    @ConfigurationProperties(prefix = "spring.video-redis")
    public RedisStandaloneConfiguration videoRedisConfig() {
        return new RedisStandaloneConfiguration();
    }

    @Bean("videoRedisFactory")
    @Primary
    public LettuceConnectionFactory factory(@Qualifier("videoRedisPool") GenericObjectPoolConfig<LettucePoolingClientConfiguration> videoRedisPool,
                                            @Qualifier("videoRedisConfig") RedisStandaloneConfiguration videoRedisConfig) {
        LettuceClientConfiguration lettuceClientConfiguration =
                LettucePoolingClientConfiguration.builder().poolConfig(videoRedisPool).build();
        return new LettuceConnectionFactory(videoRedisConfig, lettuceClientConfiguration);
    }

    @Bean(name = "videoRedisTemplate")
    public RedisTemplate<String, String> getRedisTemplate(@Qualifier("videoRedisFactory") RedisConnectionFactory factory) {
        return buildRedisTemplateByString(factory);
    }

    /**
     * 构建redisTemplate 使用string序列化
     *
     * @param factory
     * @return
     */
    private RedisTemplate<String, String> buildRedisTemplateByString(RedisConnectionFactory factory) {
        RedisTemplate<String, String> redisTemplate = new RedisTemplate<>();
        redisTemplate.setConnectionFactory(factory);
        // key的序列化类型 保证可读性
        redisTemplate.setKeySerializer(new StringRedisSerializer());
        redisTemplate.setValueSerializer(new StringRedisSerializer());
        redisTemplate.setHashKeySerializer(new StringRedisSerializer());
        redisTemplate.setHashValueSerializer(new StringRedisSerializer());
        return redisTemplate;
    }

}
